#ifndef POKEPLATINUM_UNK_02017728_H
#define POKEPLATINUM_UNK_02017728_H

#include "functypes/funcptr_02017798.h"

void sub_02017728(void);
void sub_02017798(UnkFuncPtr_02017798 param0, void * param1);
void sub_020177A4(void);
BOOL sub_020177BC(UnkFuncPtr_02017798 param0, void * param1);
void sub_0201789C(void);
void sub_020179E4(void);
void * sub_02017A40(int param0, const char * param1);
void sub_02017A94(const char * param0, void ** param1);
void sub_02017ACC(void);
void sub_02017AF4(void);
void sub_02017B70(int param0);
void sub_02017B7C(u8 param0);
void sub_02017B8C(u8 param0);
void sub_02017B9C(void);
void sub_02017DD4(int param0, int param1);
void sub_02017DE0(u8 param0);
void sub_02017DF0(u8 param0);
void sub_02017E00(int param0);
void sub_02017E2C(void);
BOOL sub_02017E54(void);

#endif // POKEPLATINUM_UNK_02017728_H
